#!/usr/bin/env python2
import os
import random
import sys
import time

# /etc/openvpn/ip6_route_verify.py
#
# script to periodically ping some site on the internet to check if IPv6
# is reachable.
# If it is, comment-in the route rule in radvd.conv and restart radvd.service
# If it is not reachable, comment-out the rule and restart radvd.


RADVD_CONF_FILE = "/etc/radvd.conf"
LOOP_DELAY = 30.0
PING_SERVERS_LIST = (
	"pengutronix.de",
	"google.com",
	"heise.de",
	"facebook.com",
	"netflix.com",
	"youtube.com",
	"yahoo.com",
	"wikipedia.org",
	"strato.de",
	"instagram.com",
	"whitehouse.gov",
	"mit.edu"
)


def touch_radvd_conf(filename,enable_route):
	# read file.
	lines = list()
	try:
		f = file(filename,"r")
	except IOError:
		return False
	try:
		lines = f.read().split("\n")
	except IOError:
		f.close()
		return False
	f.close()

	# loop and find lines with word 'route'
	i=0
	while i<len(lines):
		if lines[i].find("route ")>=0 and lines[i].find("2000::")>=0:
			# from here, find next line with a closing bracket '}'
			for j in xrange(i,len(lines)):
				if lines[j].find("}")>=0:
					break
			# comment in or out all these lines.
			for k in xrange(i,j+1):
				lin = lines[k]
				if enable_route:
					lin = lin.lstrip("#")
				elif not lin.startswith("#"):
					lin = "#"+lin
				lines[k] = lin
			# skip section
			i=j
		i+=1

	# reassemble file and write back
	try:
		f = file(filename,"w")
	except IOError:
		return False
	try:
		lines = f.write("\n".join(lines))
	except IOError:
		f.close()
		return False
	f.close()
	return True


def pingone(address):
	if os.name!="nt":
		cmd = "ping6 -c 1 -i 0.2 "+address+" >/dev/null 2>&1"
	else:
		cmd = "ping -6 -n 1 "+address
	retval = os.system(cmd)
	return retval==0

def ping_list():
	n = len(PING_SERVERS_LIST)
	startat = random.randint(0,n-1)
	for i in xrange(n):
		adr = PING_SERVERS_LIST[(startat+i)%n]
		if pingone(adr):
			print "pinged %s" % adr
			return True
	print "could not ping any on list."
	return False



# working loop
have_route = None
while True:
	# test if we have ipv6 internet
	have = ping_list()
	if have!=have_route:
		have_route = have
		res = touch_radvd_conf(RADVD_CONF_FILE,have)
		if not res:
			sys.stderr.write("ERROR: could not modify file '%s'\n"%(RADVD_CONF_FILE,))
			have_route = None
		else:
			cmd = "systemctl restart radvd.service"
			res = os.system(cmd)
			if res!=0:
				sys.stderr.write("ERROR: calling '%s' returnd %d\n"%(cmd,res))
			else:
				print "restarted radvd"
	time.sleep(LOOP_DELAY)

# matching service file for systemd to auto-start this.
"""
[Unit]
Description=IPv6 internet-route checker
After=network-online.target radvd.service
Conflicts=shutdown.target

[Service]
ExecStart=/etc/openvpn/ip6_route_verify.py
Type=simple
KillMode=process

[Install]
WantedBy=multi-user.target

"""


